"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var actions_1 = require("./actions");
var immer_1 = __importDefault(require("immer"));
exports.addEdge = immer_1.default(function (state, _a) {
    var payload = _a.payload;
    state.nodes[payload.from].edges[payload.to] = { weight: 1, name: payload.fromName };
    state.nodes[payload.to].edges[payload.from] = { weight: 1, name: payload.toName };
    state.nOfEdges += 1;
});
exports.addNode = immer_1.default(function (state, _a) {
    var payload = _a.payload;
    state.nodes[payload.key] = {
        value: payload.value || {},
        edges: payload.edges || {}
    };
    if (payload.edges) {
        Object.keys(payload.edges).forEach(function (edge) {
            state.nodes[edge].edges[payload.key] = payload.edges[edge];
            state.nOfEdges += 1;
        });
    }
    state.nOfNodes += 1;
});
exports.removeEdge = immer_1.default(function (state, _a) {
    var payload = _a.payload;
    delete state.nodes[payload.from].edges[payload.to];
    delete state.nodes[payload.to].edges[payload.from];
    state.nOfEdges -= 1;
});
exports.removeNode = immer_1.default(function (state, _a) {
    var payload = _a.payload;
    var edges = state.nodes[payload.key].edges;
    Object.keys(edges).forEach(function (edge) {
        delete state.nodes[edge].edges[payload.key];
    });
    delete state.nodes[payload.key];
    state.nOfNodes -= 1;
});
exports.reducerMap = new Map([
    [actions_1.GraphActions.ADD_EDGE, exports.addEdge],
    [actions_1.GraphActions.ADD_NODE, exports.addNode],
    [actions_1.GraphActions.REMOVE_EDGE, exports.removeEdge],
    [actions_1.GraphActions.REMOVE_NODE, exports.removeNode]
]);
