import { Store, select } from '@reactive-redux/store';
import { IGraph, reducerMap } from './reducers';
import { AddNode, AddEdge, RemoveEdge, RemoveNode } from './actions';
import { map } from 'rxjs/operators';
import { Node, Edge } from './models';
import { Observable } from 'rxjs';

export class Graph extends Store<IGraph> {
  static readonly initialBlank: IGraph = {
    nodes: {},
    nOfNodes: 0,
    nOfEdges: 0
  };

  public state$: Observable<IGraph>;
  public nodeCount$ = this.state$.pipe(select(state => state.nOfNodes));
  public edgeCount$ = this.state$.pipe(select(state => state.nOfEdges));
  public keys$ = this.state$.pipe(select(state => Object.keys(state.nodes)));
  public values$ = this.state$.pipe(
    select(state => Object.keys(state.nodes).map(key => ({ key, ...state.nodes[key] })))
  );

  constructor(private _initialValue: IGraph = Graph.initialBlank) {
    super(_initialValue, reducerMap);
  }

  addNode(node: Node) {
    this.dispatch(new AddNode(node));
  }

  removeNode(key: string) {
    this.dispatch(new RemoveNode({ key }));
  }

  addEdge(edge: Edge) {
    this.dispatch(new AddEdge(edge));
  }

  removeEdge(from: string, to: string) {
    this.dispatch(new RemoveEdge({ from, to }));
  }

  getValue$(key: string) {
    return this.state$.pipe(
      select(state => state.nodes[key]),
      map(e => (e ? e.value : undefined))
    );
  }

  getEdges$(key: string) {
    return this.state$.pipe(
      select(state => state.nodes[key]),
      map(
        e =>
          e && e.edges ? Object.keys(e.edges).map(target => ({ target, ...e.edges[target] })) : []
      )
    );
  }

  isConnected$(keyA: string, keyB: string) {
    return this.state$.pipe(
      select(state => state.nodes[keyA]),
      map(n => !!n.edges[keyB])
    );
  }
}
