import { GraphActions, AddEdge, AddNode, RemoveEdge, RemoveNode } from './actions';
import produce from 'immer';

export interface IGraph {
  nodes: {
    [key: string]: {
      value?: any;
      edges?: { [key: string]: { weight: number; name: string } };
    };
  };
  nOfNodes: number;
  nOfEdges: number;
}

export const addEdge = produce<IGraph>((state: IGraph, { payload }: AddEdge) => {
  state.nodes[payload.from].edges[payload.to] = { weight: 1, name: payload.fromName };
  state.nodes[payload.to].edges[payload.from] = { weight: 1, name: payload.toName };

  state.nOfEdges += 1;
});

export const addNode = produce<IGraph>((state: IGraph, { payload }: AddNode) => {
  state.nodes[payload.key] = {
    value: payload.value || {},
    edges: payload.edges || {}
  };

  if (payload.edges) {
    Object.keys(payload.edges).forEach(edge => {
      state.nodes[edge].edges[payload.key] = payload.edges[edge];
      state.nOfEdges += 1;
    });
  }

  state.nOfNodes += 1;
});

export const removeEdge = produce<IGraph>((state: IGraph, { payload }: RemoveEdge) => {
  delete state.nodes[payload.from].edges[payload.to];
  delete state.nodes[payload.to].edges[payload.from];

  state.nOfEdges -= 1;
});

export const removeNode = produce<IGraph>((state: IGraph, { payload }: RemoveNode) => {
  const { edges } = state.nodes[payload.key];

  Object.keys(edges).forEach(edge => {
    delete state.nodes[edge].edges[payload.key];
  });

  delete state.nodes[payload.key];

  state.nOfNodes -= 1;
});

export const reducerMap = new Map<any>([
  [GraphActions.ADD_EDGE, addEdge],
  [GraphActions.ADD_NODE, addNode],
  [GraphActions.REMOVE_EDGE, removeEdge],
  [GraphActions.REMOVE_NODE, removeNode]
]);
